using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using JasperSite.Areas.Admin.ViewModels;
using JasperSite.Models.Database;
using JasperSite.Models;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.AspNetCore.Routing;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using JasperSite.Helpers;

namespace JasperSite.Areas.Admin.Controllers
{    
    [Authorize]
    [Area("Admin")]
    public class ArticlesController : Controller
    {
        private readonly DatabaseContext databaseContext;
        private readonly DbHelper dbHelper;

        public ArticlesController(DatabaseContext dbContext)
        {
            this.databaseContext = dbContext;
            this.dbHelper = new DbHelper(dbContext);
        }

        public ArticlesViewModel UpdatePage()
        {
            ArticlesViewModel model = new ArticlesViewModel();

            try
            {
                List<Article> articles = dbHelper.GetAllArticles();
                if (articles.Count > 0)
                    model.Articles = articles;
                else model.Articles = null;


                // Paging section --> first page, items per page
                model.ArticleListModel = UpdateArticleListModel(1, 10);

            }
            catch
            {
                model.Articles = null;
            }
            
            try
            {
                model.NumberOfCategories = dbHelper.GetAllCategories().Count();
            }
            catch
            {
                model.NumberOfCategories = 0;
            }
            
            try
            {
                model.NumberOfArticles = model.Articles.Count();
            }
            catch
            {
                model.NumberOfArticles = 0;
            }

            try
            {
               model.UncategorizedCategoryExists= dbHelper.UncategorizedCategoryExists();
            }
            catch
            {
                model.UncategorizedCategoryExists = default(bool);
            }
        
            return model;
        }

        [HttpGet]
        public IActionResult Index()
        {
            return View(UpdatePage());
        }

        public ArticleListViewModel UpdateArticleListModel(int currentPage, int itemsPerPage)
        {
            ArticleListViewModel modelOutput = new ArticleListViewModel();
            List<Article> articleList = dbHelper.GetAllArticles();
            articleList.OrderBy(a => a.Name);

            JasperPaging<Article> paging = new JasperPaging<Article>(articleList, currentPage, itemsPerPage);

            modelOutput.CurrentPage = paging.CurrentPageNumber;
            modelOutput.TotalNumberOfPages = paging.NumberOfPagesNeeded;
            modelOutput.Articles = paging.GetCurrentPageItems();
            return modelOutput;

        }

        [HttpPost]
        public IActionResult ArticlePaging(ArticleListViewModel model, IFormCollection collection)
        {

            ArticleListViewModel modelOutput = new ArticleListViewModel();

            try
            {
                int itemsPerPage = 10;
                int currentPage = model.CurrentPage;

                string next = collection["next"];
                string prev = collection["prev"];

                if (next != null)
                    currentPage++;
                if (prev != null)
                    currentPage--;

                modelOutput = UpdateArticleListModel(currentPage, itemsPerPage);

            }
            catch
            {
                TempData["ErrorMessage"] = "Pi provdn poadavku dolo k chyb";
            }

            bool isAjaxRequest = Request.Headers["x-requested-with"] == "XMLHttpRequest";

            if (isAjaxRequest)
            {
                ModelState.Clear();
                return PartialView("ArticlesListPartialView", modelOutput);
            }
            else
            {
                return RedirectToAction("Index");
            }
        }

        [HttpGet]
        // Returns only articles with Published=true
        public IActionResult GetArticle(int id)
        {
            try
            {
                EditArticleViewModel model = GetEditArticleModel(id);

                if (model == null || id <= 0)
                {
                    throw new Exception();
                }

                return View("ArticleEdit", model);
            }
            catch
            {
                TempData["ErrorMessage"] = "Dan lnek nebyl nalezen";
                return RedirectToAction("Index");
            }           

         
        }

        public EditArticleViewModel GetEditArticleModel(int id)
        {
            try
            {
                Article articleToEdit = dbHelper.GetArticleById(id);

                //if (!articleToEdit.Publish) return null; // not published article will not be served

                EditArticleViewModel model = new EditArticleViewModel
                {
                    Id = articleToEdit.Id,
                    HtmlContent = articleToEdit.HtmlContent,
                    Name = articleToEdit.Name,
                    PublishDate = articleToEdit.PublishDate,
                    Categories = dbHelper.GetAllCategories(),
                    SelectedCategoryId = articleToEdit.CategoryId,
                    Publish = articleToEdit.Publish,
                    Keywords = articleToEdit.Keywords                    
                    
                };

                // URL rewriting
                model.AllUrl = dbHelper.GetUrls(articleToEdit.Id);
                model.ArticlesRoute = Configuration.WebsiteConfig.ArticleRoute;

                return model;
            }
            catch
            {
                return null;
            }
            
        }

        // This action method does not return partial view - too slow and laggy for tinyMCE
        [HttpPost]
        public IActionResult PostArticle(EditArticleViewModel model)
        {
            bool isAjax = HttpContext.Request.Headers["X-Requested-With"] == "XMLHttpRequest";

            try
            {
                if (ModelState.IsValid)
                {
                    Article newArticleData = new Article();
                    newArticleData.Id = model.Id;
                    newArticleData.HtmlContent = model.HtmlContent;
                    newArticleData.Name = model.Name;
                    newArticleData.PublishDate = (DateTime)model.PublishDate;
                    newArticleData.CategoryId = model.SelectedCategoryId;
                    newArticleData.Publish = model.Publish;
                    newArticleData.Keywords = model.Keywords;
                    Article articleReference = dbHelper.EditArticle(newArticleData);

                    // URL rewriting

                    if (!string.IsNullOrEmpty(model.Url))
                    {                      
                       dbHelper.SetUrl(articleReference, model.Url);                        
                    }

                    // At least one ULR rewriting rule has to exist and URL rewriting has to be allowed
                    int numberOfRules = dbHelper.GetUrls(model.Id).Count();
                    if(numberOfRules<1 && Configuration.WebsiteConfig.UrlRewriting)
                    {
                        throw new NoUrlRulesException("At least one rewriting rule has to be present.");
                    }

                    TempData["Success"] = true;

                }
                else
                {
                    throw new Exception();
                }
            }
            catch (NoUrlRulesException)
            {
                TempData["ErrorMessage"] = "Je nutn piadit alespo jednu URL adresu.";
            }
            catch(InvalidUrlRewriteException ex)
            {
                try
                {
                    string alreadyAssignedAricleName = dbHelper.GetArticleById(ex.AssignedArticleId).Name;
                    TempData["ErrorMessage"] = "Zadan URL adresa je ji obsazen lnkem: "+alreadyAssignedAricleName+ ".";
                }
                catch 
                {                    
                    TempData["ErrorMessage"] = "Zadan URL adresa je ji obsazen.";
                }
            }
            catch (Exception)
            {
               
                TempData["ErrorMessage"] = "Zmny nebylo mon uloit.";
            }

            if (isAjax)
            {
                // return Content(string.Empty); 

                // Return URL routes only for current article
                UrlListViewModel partialModel = new UrlListViewModel();
                partialModel.Urls = dbHelper.GetAllUrls().Where(url => url.ArticleId == model.Id).Select(u => u.Url).ToList();
                partialModel.ArticleId = model.Id;

              return PartialView("UrlListPartialView", partialModel);
            }
            else
            {
                return RedirectToAction("GetArticle", new { id = model.Id });
            }

        }

        [HttpGet]
        public IActionResult Add()
        {
            try
            {                
                int articleId = dbHelper.AddArticle().Id;
                return RedirectToAction("GetArticle", new { id = articleId });
            }
            catch(Exception)
            {
                TempData["ErrorMessage"] = "Dan lnek nebylo mon vytvoit";

                return RedirectToAction("Index");
                
            }        
        }

        [HttpGet]
        public IActionResult Delete(int id, int page)
        {
            bool isAjax = HttpContext.Request.Headers["X-Requested-With"] == "XMLHttpRequest";

            try
            {
                dbHelper.DeleteArticle(id);
                TempData["Success"] = true;
            }
            catch (Exception)
            {
                TempData["ErrorMessage"] = "Dan lnek nebylo mon odstranit";
            }

            
            if (isAjax)
            {
                return PartialView("ArticlesListPartialView", UpdateArticleListModel(page,10)); 
            }
            else
            {
                return RedirectToAction("Index");
            }
        }

        [HttpGet]
        public IActionResult CreateUncategorizedCategory()
        {
            try
            {
                dbHelper.CreateUncategorizedCategory();
                TempData["Success"] = true;
            }
            catch(Exception)
            {
                TempData["ErrorMessage"] = "Kategorii \"nezaazeno\" se nepodailo vytvoit.";                
            }
            
            return RedirectToAction("Index");
        }

       
        [HttpGet]
        public IActionResult DeleteRoute(string name, int? currentArticleId)
        {
            bool isAjax = HttpContext.Request.Headers["X-Requested-With"] == "XMLHttpRequest";

            try
            {
                // Both parameters has to be filled in
                if (string.IsNullOrEmpty(name) || currentArticleId == null)
                {
                    TempData["ErrorMessage"] = "Poloku se nepodailo odstranit.";
                    return RedirectToAction("Index");
                }
                else
                {
                    // Let's test if article and URL exists
                    try
                    {
                        Article article = dbHelper.GetArticleById((int)currentArticleId);
                        UrlRewrite rule= dbHelper.GetAllUrls().Where(url => url.Url == name).Single();
                    }
                    catch
                    {
                        TempData["ErrorMessage"] = "Poloku se nepodailo odstranit.";
                        return RedirectToAction("Index");
                    }
                   
                    
                }

                // At least one ULR rewriting rule has to exist and URL rewriting has to be allowed
                int numberOfRules = dbHelper.GetUrls((int)currentArticleId).Count();                                               

                if (numberOfRules < 2 && Configuration.WebsiteConfig.UrlRewriting)
                {
                    throw new NoUrlRulesException("At least one rewriting rule has to be present.");
                }

                dbHelper.DeleteUrl(name);

                TempData["Success"] = true;
                  

            }
            catch(InvalidUrlRewriteException)
            {
                TempData["ErrorMessage"] = "Dan URL adresa nebyla nalezena pro operaci odstrann.";
            }
            catch (NoUrlRulesException)
            {
                TempData["ErrorMessage"] = "Pro dan lnek mus existovat alespo jedna URL adresa.";
            }
            catch (Exception)
            {
                TempData["ErrorMessage"] = "Poloku se nepodailo odstranit";
            }


            if (isAjax)
            {
                              
                    // Return URL routes only for current article
                    UrlListViewModel model = new UrlListViewModel();
                    model.Urls = dbHelper.GetUrls((int)currentArticleId);
                    model.ArticleId = (int)currentArticleId;
                    return PartialView("UrlListPartialView", model); 
                
            }
            else
            {
                return RedirectToAction("Index");
            }


        }

    }
}